@@ -42,23 +42,8 @@ extern "C" {
  */
 #include <gensio/gensio_os_funcs_public.h>
 
-/*
- * A bitmask of log levels to tell what to log.  Defaults to fatal and err
- * only.
- */
-GENSIO_DLL_PUBLIC
-void gensio_set_log_mask(unsigned int mask);
-GENSIO_DLL_PUBLIC
-unsigned int gensio_get_log_mask(void);
-GENSIO_DLL_PUBLIC
-const char *gensio_log_level_to_str(enum gensio_log_levels level);
-
-GENSIO_DLL_PUBLIC
-void gensio_vlog(struct gensio_os_funcs *o, enum gensio_log_levels level,
-		 const char *str, va_list args);
-GENSIO_DLL_PUBLIC
-void gensio_log(struct gensio_os_funcs *o, enum gensio_log_levels level,
-		const char *str, ...);
+#include <gensio/gensio_utils.h>
+#include <gensio/gensio_addr.h>
 
 /*
  * The following are documented in gensio_event.3
@@ -75,6 +60,12 @@ void gensio_log(struct gensio_os_funcs *o, enum gensio_log_levels level,
 #define GENSIO_EVENT_REQUEST_2FA	10
 #define GENSIO_EVENT_2FA_VERIFY		11
 
+struct gensio_parmlog_data {
+    const char *log;
+    va_list args;
+};
+#define GENSIO_EVENT_PARMLOG		12
+
 /*
  * Serial callbacks start here and run to 2000.
  */
@@ -94,6 +85,21 @@ int str_to_gensio(const char *str,
 		  gensio_event cb, void *user_data,
 		  struct gensio **gensio);
 
+GENSIO_DLL_PUBLIC
+int gensio_terminal_alloc(const char *gensiotype, const void *gdata,
+			  const char * const args[],
+			  struct gensio_os_funcs *o,
+			  gensio_event cb, void *user_data,
+			  struct gensio **new_gensio);
+
+GENSIO_DLL_PUBLIC
+int gensio_filter_alloc(const char *gensiotype,
+			struct gensio *child,
+			const char * const args[],
+			struct gensio_os_funcs *o,
+			gensio_event cb, void *user_data,
+			struct gensio **new_gensio);
+
 GENSIO_DLL_PUBLIC
 int str_to_gensio_child(struct gensio *child, const char *str,
 			struct gensio_os_funcs *o,
@@ -279,11 +285,28 @@ struct gensio_acc_postcert_verify_data {
 #define GENSIO_ACC_EVENT_REQUEST_2FA		9
 /* Uses struct gensio_acc_password_verify_data */
 
+#define GENSIO_ACC_EVENT_PARMLOG		10
+
 GENSIO_DLL_PUBLIC
 int str_to_gensio_accepter(const char *str, struct gensio_os_funcs *o,
 			   gensio_accepter_event cb, void *user_data,
 			   struct gensio_accepter **accepter);
 
+GENSIO_DLL_PUBLIC
+int gensio_terminal_acc_alloc(const char *gensiotype, const void *gdata,
+			      const char * const args[],
+			      struct gensio_os_funcs *o,
+			      gensio_accepter_event cb, void *user_data,
+			      struct gensio_accepter **accepter);
+
+GENSIO_DLL_PUBLIC
+int gensio_filter_acc_alloc(const char *gensiotype,
+			    struct gensio_accepter *child,
+			    const char * const args[],
+			    struct gensio_os_funcs *o,
+			    gensio_accepter_event cb, void *user_data,
+			    struct gensio_accepter **accepter);
+
 GENSIO_DLL_PUBLIC
 int str_to_gensio_accepter_child(struct gensio_accepter *child,
 				 const char *str,
@@ -387,186 +410,6 @@ void gensio_acc_set_frdata(struct gensio_accepter *acc,
 GENSIO_DLL_PUBLIC
 struct gensio_acc_frdata *gensio_acc_get_frdata(struct gensio_accepter *acc);
 
-/*
- * These are the low-level network protocol that gensio support.  Used
- * mostly in interacting with addresses, anything named protocol.
- * zero is reserved.
- */
-#define GENSIO_NET_PROTOCOL_TCP 1
-#define GENSIO_NET_PROTOCOL_UDP 2
-#define GENSIO_NET_PROTOCOL_SCTP 3
-#define GENSIO_NET_PROTOCOL_UNIX 4
-
-/*
- * Dealing with iterators.
- */
-GENSIO_DLL_PUBLIC
-void gensio_addr_rewind(struct gensio_addr *addr);
-/* Return false if no more addresses exist. */
-GENSIO_DLL_PUBLIC
-bool gensio_addr_next(struct gensio_addr *addr);
-/*
- * Gets the current address.  len must be provided, it is the size of
- * the buffer and is updated to the actual size (which may be larger
- * than len).  The copy may be partial if len is not enough.
- */
-GENSIO_DLL_PUBLIC
-void gensio_addr_getaddr(const struct gensio_addr *addr,
-			 void *oaddr, gensiods *len);
-
-#define GENSIO_NETTYPE_UNSPEC	0
-#define GENSIO_NETTYPE_IPV4	1
-#define GENSIO_NETTYPE_IPV6	2
-#define GENSIO_NETTYPE_UNIX	3
-#define GENSIO_NETTYPE_AX25	4
-
-/*
- * Create a gensio address from raw address data.  Note that the iaddr
- * data is type in_addr for ipv4, in6_addr for ipv6, and the patch for
- * unix.
- */
-GENSIO_DLL_PUBLIC
-int gensio_addr_create(struct gensio_os_funcs *o,
-		       int nettype, const void *iaddr, gensiods len,
-		       unsigned int port, struct gensio_addr **newaddr);
-
-/*
- * Return the network type (ipv4, ipv6, unix socket, etc.) for the
- * current address.
- */
-GENSIO_DLL_PUBLIC
-int gensio_addr_get_nettype(const struct gensio_addr *addr);
-
-/*
- * If the address can be supported by a socket with the given
- * family/flags combo, return true.  This will return true if the
- * families match or if address ipv4, family is IPv6, and flags has
- * AI_V4MAPPED.
- */
-GENSIO_DLL_PUBLIC
-bool gensio_addr_family_supports(const struct gensio_addr *addr, int family,
-				 int flags);
-
-/*
- * A routine for converting a current address to a string representation
- *
- * The output is put into buf starting at *epos (or zero if epos is NULL)
- * and will fill in buf up to buf + buflen.  If the buffer is not large
- * enough, it is truncated, but if epos is not NULL, it will be set to the
- * byte position where the ending NIL character would have been, one less
- * than the buflen that would have been required to hold the entire buffer.
- */
-GENSIO_DLL_PUBLIC
-int gensio_addr_to_str(const struct gensio_addr *addr,
-		       char *buf, gensiods *epos, gensiods buflen);
-
-/*
- * Like the above, but does all the addresses, not just the current
- * one, separated by ';'.
- */
-GENSIO_DLL_PUBLIC
-int gensio_addr_to_str_all(const struct gensio_addr *addr,
-			   char *buf, gensiods *epos, gensiods buflen);
-
-/*
- * Compare two addresses and return TRUE if they are equal and FALSE
- * if not.  If compare_ports is false, then the port comparison is
- * ignored.
- *
- * If compare_all is true, verify that all the addresses are the same.
- * If it is false, only compare the current address.
- */
-GENSIO_DLL_PUBLIC
-bool gensio_addr_equal(const struct gensio_addr *a1,
-		       const struct gensio_addr *a2,
-		       bool compare_ports, bool compare_all);
-
-/*
- * Create a new address structure with the same addresses.
- */
-GENSIO_DLL_PUBLIC
-struct gensio_addr *gensio_addr_dup(const struct gensio_addr *ai);
-
-/*
- * Concatenate two addr structures and return a new one.
- */
-GENSIO_DLL_PUBLIC
-struct gensio_addr *gensio_addr_cat(const struct gensio_addr *ai1,
-				    const struct gensio_addr *ai2);
-
-/*
- * Decrement the refcount on the structure and free if not in use.
- */
-GENSIO_DLL_PUBLIC
-void gensio_addr_free(struct gensio_addr *ai);
-
-/*
- * See if addr is present in ai.  Ports are not compared unless
- * compare_ports is true.
- */
-GENSIO_DLL_PUBLIC
-bool gensio_addr_addr_present(const struct gensio_addr *ai,
-			      const void *addr, gensiods addrlen,
-			      bool compare_ports);
-
-/*
- * Scan for a network port in the form:
- *
- *   <ipspec><protocol>,<hostnames>
- *
- *   protocol = [tcp|udp|sctp|unix[(<args>)]]
- *
- * for unix:
- *   hostnames = <file path>
- *
- * for others:
- *   hostnames = [[...]<ipspec>[<hostname>,]<port>,]<ipspec>[<hostname>,]<port>
- *
- *   ipspec = [ipv4|ipv6|ipv6n4,]
- *
- * ipspec is not allowed with unix.
- *
- * The initial ipspec sets the default for all the addresses.  If it
- * is not specified, the default if AF_UNSPEC and everything will
- * be returned.
- *
- * If a protocol is not specified, the TCP is assumed.
- *
- * If the args parameter supplied is NULL, then you cannot specify
- * args in the string, EINVAL will be returned.
- *
- * You can specify the IP address type on each hostname/port and it
- * overrides the default.  The hostname can be a resolvable hostname,
- * an IPv4 octet, an IPv6 address, or an empty string.  If it is not
- * supplied, inaddr_any is used.  In the absence of a hostname
- * specification, a wildcard address is used.  The mandatory second
- * part is the port number or a service name.
- *
- * An all zero port means use any port. If the port is all zero on any
- * address, then is_port_set is set to false, true otherwise.
- *
- * The protocol type is returned, either TCP, UDP, or SCTP.  Protocol
- * may be NULL.
- *
- * ai should be freed with gensio_addr_free().
- *
- * args should be freed with str_to_argv_free().
- */
-GENSIO_DLL_PUBLIC
-int gensio_scan_network_port(struct gensio_os_funcs *o, const char *str,
-			     bool listen, struct gensio_addr **ai,
-			     int *protocol, bool *is_port_set,
-			     int *argc, const char ***args);
-
-/*
- * Like the above, but only scan for addresses in a list, no ports, no
- * protocol, like: "::1,ipv4,10.0.2.3".  This only works on IP
- * addresses.
- */
-GENSIO_DLL_PUBLIC
-int gensio_scan_network_addr(struct gensio_os_funcs *o, const char *str,
-			     int protocol, struct gensio_addr **ai);
-
 /*
  * Handling for gensio parameters.
  */
@@ -578,11 +421,6 @@ enum gensio_default_type {
     GENSIO_DEFAULT_DATA
 };
 
-struct gensio_enum_val {
-    char *name;
-    int val;
-};
-
 GENSIO_DLL_PUBLIC
 int gensio_add_default(struct gensio_os_funcs *o,
 		       const char *name,
@@ -674,84 +512,22 @@ GENSIO_DLL_PUBLIC
 int gensio_check_keymode(const char *str, const char *key, unsigned int *rmode);
 GENSIO_DLL_PUBLIC
 int gensio_check_keyperm(const char *str, const char *key, unsigned int *rmode);
-
 /*
- * Helper functions that don't fit anywhere else.
- */
-
-/*
- * Returns true of str is in one of auxdata, false if not.
+ * Get a gensio time structure. Time consists of a set of numbers each
+ * followed by a single letter.  That letter may be 'D', 'H', 'M',
+ * 's', 'm', 'u', or 'n', meaning days, hours, minutes, seconds,
+ * milliseconds, microseconds, or nanoseconds.  So, for instance,
+ * "10D5H4M9s100u" would be ten days, 5 hours, 4 minutes, 9 seconds,
+ * 100 microseconds.  If a plain number with no letter at the end is
+ * given, then the value passed in "mod" is used.  Pass in 0 for "mod"
+ * to require the user to specify the modifier.
  */
 GENSIO_DLL_PUBLIC
-bool gensio_str_in_auxdata(const char *const *auxdata, const char *str);
-
-/*
- * Set the program name, used by TCPD (and possibly others).  You should
- * do this very early in initialization, first if possible.  The default
- * progname is "gensio".
- *
- * The string is *NOT* copied, so you must make sure it stays around.
- * Generally you are passing in a constant string or part of argv[0],
- * so it's not a problem
- */
-GENSIO_DLL_PUBLIC
-bool gensio_set_progname(const char *progname);
-
-/*
- * Various conversion helpers.  These may become inline someday...
- */
-GENSIO_DLL_PUBLIC
-uint32_t gensio_buf_to_u32(unsigned char *data);
-GENSIO_DLL_PUBLIC
-void gensio_u32_to_buf(unsigned char *data, uint32_t v);
-GENSIO_DLL_PUBLIC
-uint16_t gensio_buf_to_u16(unsigned char *data);
-GENSIO_DLL_PUBLIC
-void gensio_u16_to_buf(unsigned char *data, uint16_t v);
-
-/*
- * A helper function, very useful for raddr handling.  Do an
- * snprintf() at buf + *pos, writing to up to buf + len.  If *pos > len,
- * then don't do anything, but always return the number of characters
- * that would have been output if there was enough room. Pos is updated
- * to the new location it would have been if there was enough room.
- */
-GENSIO_DLL_PUBLIC
-gensiods gensio_pos_snprintf(char *buf, gensiods len, gensiods *pos,
-			     char *format, ...);
-
-/*
- * Like the above, but it handles converting an argv to a string, properly
- * quoting everything.
- */
-GENSIO_DLL_PUBLIC
-gensiods gensio_argv_snprintf(char *buf, gensiods len, gensiods *pos,
-			      const char **argv);
-
-/*
- * An sprintf that allocates the memory
- */
-GENSIO_DLL_PUBLIC
-char *gensio_alloc_vsprintf(struct gensio_os_funcs *o,
-			    const char *fmt, va_list va);
-GENSIO_DLL_PUBLIC
-char *gensio_alloc_sprintf(struct gensio_os_funcs *o,
-			   const char *fmt, ...);
+int gensio_check_keytime(const char *str, const char *key, char mod,
+			 gensio_time *rgt);
 
 GENSIO_DLL_PUBLIC
-char *gensio_strdup(struct gensio_os_funcs *o, const char *str);
-
-GENSIO_DLL_PUBLIC
-char *gensio_strndup(struct gensio_os_funcs *o, const char *str, gensiods len);
-
-/*
- * Take the input string, put " around it, and put a \ infront of
- * every \ and ".  This allows you to take a string with " and \ in it
- * and have them pass through the str_to_gensio() and such functions
- * properly.
- */
-GENSIO_DLL_PUBLIC
-char *gensio_quote_string(struct gensio_os_funcs *o, const char *str);
+int gensio_check_keyfloat(const char *str, const char *key, float *rfl);
 
 /*
  * Return the number of allocated gensios.  This is primarily for
@@ -760,10 +536,6 @@ char *gensio_quote_string(struct gensio_os_funcs *o, const char *str);
 GENSIO_DLL_PUBLIC
 gensiods gensio_num_alloced(void);
 
-/* For testing, do not use in normal code. */
-GENSIO_DLL_PUBLIC
-void gensio_osfunc_exit(int rv);
-
 /*
  * Generic functions for dumping buffer data to stdio
  */
